function [ stump, trainError ] = decisionstump_fit( X, T, w, nSplits )
%DECISIONSTUMP_FIT Summary of this function goes here
%   X - Nxdim matrix, N samples with dim properties each
%   T - Nx1 vector, the true labels of the observations (assumes T \in {-1,+1})
%   w - Nx1 vector, the samples weights
%   nSplits - how many split values to try out
%
%   stump.dim       - the best dimension (feature) to split
%   stump.value     - the decision value on which to split
%   stump.leftLabel - the class label of the samples that are left of the decision value 


if ~exist('nSplits', 'var'), nSplits = 100; end

dim = size(X,2);
minErrors = inf(dim,1); % error storage
bestSplits = zeros(dim,1); % split value storage
leftLabels = zeros(dim,1); % priorities storage

% For each dimension in feature space
% This can be parallelized
for d=1:dim

    [minErrors(d), bestSplits(d), leftLabels(d)] = decisionstump_fit_dim(X(:,d), T, w, nSplits);

end

% Choose the best dimension 
[trainError, stump.dim] = min(minErrors);
stump.value = bestSplits(stump.dim);
stump.leftLabel = leftLabels(stump.dim);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% A very simple "Decision Stump" weak classifier
function [minError, bestValue, leftLabel] = decisionstump_fit_dim(X, T, w, nSplits)
%DECISIONSTUMP_FIT_DIM Summary of this function goes here
%   X - Nx1 vector, one dimension of N samples
%   T - Nx1 vector, the true labels of the observations (assumes T \in {-1,+1})
%   w - Nx1 vector, the samples weights
%   nSplits - how many split values to try out
%
%   minError        - the minimum error found
%   bestValue       - the decision value that gave the minimum error
%   leftLabel       - the class label of the samples left of the decision value 

    N = length(X);
    Xmin = min(X);
    Xrange = range(X);
    Xmax = Xmin + Xrange;
    step = Xrange*nSplits; 
    x = sort(X);
    
    labels = unique(T);
    
    minError = inf;
    stumpDim.dim = 1;

    % Find an axis aligned hyperplane that minimizes the classification error
    for i=1:(N-1)

        stumpDim.value = (x(i) + x(i+1))/2;
        
        stumpDim.leftLabel = labels(1);
        Pred = decisionstump_predict(X, stumpDim);
        errorIdx = (Pred ~= T);
        Jm = sum(w .* errorIdx);  

        if (Jm < minError)
            minError = Jm;
            bestValue = stumpDim.value;
            leftLabel = labels(2);
        end

        % Also check for inverted labels         
        stumpDim.leftLabel = labels(2);
        Pred = decisionstump_predict(X, stumpDim);      
        errorIdx = (Pred ~= T);
        Jm = sum(w .* errorIdx);    

        if (Jm < minError)
            minError = Jm;
            bestValue = stumpDim.value;
            leftLabel = labels(1);
        end

    end
   
end


end

